function [ boolean ] = ascent( p_nemo_data, p_config )
%ASCENT Generates plots showing the ascent data of the float.
%   Generates a plot for every float in p_nemo_data containing the depth on
%   recorded time of day, profile recording time on profile number as well
%   as 0 to 50m depth on profile number.
%
%   Parameters:
%   p_nemo_data:            The float data.
%   p_config:               The nemo configuration.
%
%   Returns:
%   boolean:                True if successful.

%% Initialize return variable
boolean = false;

%% Parameter check %TODO

%% Get config parameter
% get plot details
width = p_config.PLOT_ASCENT.image_width;
height = p_config.PLOT_ASCENT.image_height;
paperunit = p_config.PLOT_ASCENT.image_paperunit;
image_type = p_config.PLOT_ASCENT.image_type;
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ASCENT.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ASCENT.storage_directory_fig);
ylabel_fontsize = p_config.PLOT_ASCENT.ylabel_fontsize;
xlabel_fontsize = p_config.PLOT_ASCENT.ylabel_fontsize;
axis_fontsize = p_config.PLOT_ASCENT.axis_fontsize;
title_fontsize = p_config.PLOT_ASCENT.title_fontsize;

if (~isdir(directory))
    mkdir(directory);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Collect required data

switch (class(p_nemo_data{1}))
    case 'struct' % only one float is to be processed
        boolean = plot_float(p_nemo_data);
    case 'cell' % more than one float has to be processed
        boolean = all(cellfun(@plot_float, p_nemo_data));
    otherwise
        disp([mfilename ': Class ' class(p_nemo_data{1}) ' not supported for plotting!']);
end

    function boolean = plot_float(p_float)
        boolean = false;
        profile_data = nemo.float.collect(p_float, 'PROFILE_DATA', '');
        profile_best_guess_datevec = nemo.float.collect(p_float, ...
                                               'SECOND_ORDER_INFORMATION', ...
                                               'best_guess_datevec');
%         profiles_best_guess_datenum = datenum([1970, 1, 1, profiles_best_guess_datevec(:, 4:6)]);
        if (isempty(profile_data))
            boolean = true;
            return
        end
        
        % initialize figure
        figure_id = figure('Visible', 'Off');
        set(figure_id,'InvertHardcopy','on');
        set(figure_id,'PaperUnits', paperunit, 'Position', [ 0 0 width height]);
        papersize = get(figure_id, 'PaperSize');
        left = (papersize(1) - width)/2;
        bottom = (papersize(2) - height)/2;
        myfiguresize = [left, bottom, width, height];
        set(figure_id,'PaperPosition', myfiguresize);
        
        % Depth on time plot
        sub(1) = subplot(2, 2, 1);
        hold on
        set(gca, 'YLimMode', 'manual', 'YLim', [0 2000], ...
            'YTick', [0:200:2000], 'YDir', 'reverse');
        xlimits = [datenum(0, 0, 0, 0, 0, 0) datenum(0, 0, 0, 24, 0, 0)];
        set(gca, 'XLim', xlimits, 'XTick', linspace(xlimits(1), xlimits(2), 7));
        for i = 1:length(profile_data)
            plot_x = mod(datenum([ ...
                                  repmat(1970, length(profile_data{i}), 1) ...
                                  , ones(length(profile_data{i}), 1) ...
                                  , ones(length(profile_data{i}), 1) ...
                                  , profile_data{i}(:, 2:4)]), 1);
            plot_y = profile_data{i}(:, 7);
            plot_c = [0 0 1] * i/length(profile_data);
            scatter(plot_x, plot_y, 30, plot_c, 'filled');
        end
        datetick('x','HH:MM', 'keepticks', 'keeplimits');
        a = get(gca, 'YTickLabel');
        set(gca, 'YTickLabel', a, 'fontsize', axis_fontsize)
        a = get(gca, 'XTickLabel');
        set(gca, 'XTickLabel', a, 'fontsize', axis_fontsize)
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        xlabel('Time of day', 'FontSize', xlabel_fontsize);
        box on
        grid on
        hold off
        
        % Profile data time on profile number
        sub(2) = subplot(2, 2, 3);
        hold on
        ylimits = [datenum(0, 0, 0, 0, 0, 0) datenum(0, 0, 0, 24, 0, 0)];
        set(gca, 'YLimMode', 'manual', 'YLim', ylimits, 'YTick', linspace(ylimits(1), ylimits(2), 7));
        % get required information about x-axis limits and ticks
        profile_best_guess_datenum = datenum(profile_best_guess_datevec);
%         start_datenum = min(profile_best_guess_datenum);
%         end_datenum = max(profile_best_guess_datenum);
%         x_diff = max(profile_best_guess_datenum) ...
%                      - min(profile_best_guess_datenum);
%         if (start_datenum ~= end_datenum)
%             set(gca, ...
%                 'XLimMode', 'manual', ...
%                 'XLim', [start_datenum, end_datenum]);
%         end
%         if (x_diff < 30)
%             set(gca, 'XTick', [start_datenum:5:end_datenum]);
%         elseif (x_diff < 365)
%             set(gca, 'XTick', [start_datenum:30:end_datenum]);
%         else
%             set(gca, 'XTick', [start_datenum:365:end_datenum]);
%         end
        for i = 1:length(profile_data)
            plot_x = ones(1, length(profile_data{i}(:, 1))) * i;
            plot_y = datenum([zeros(length(profile_data{i}(:, 1)), 1), ...
                              zeros(length(profile_data{i}(:, 1)), 1), ...
                              zeros(length(profile_data{i}(:, 1)), 1), ...
                              profile_data{i}(:,2:4)]);
            scatter(plot_x, plot_y, 30, [0 0 0], 'filled');
        end
        datetick('y','HH:MM', 'keepticks', 'keeplimits');
%         datetick('x','yyyy-mm-dd', 'keepticks', 'keeplimits');
        set(gca, 'fontsize', axis_fontsize)
        ylabel('Profile recording time [HH:MM]', 'FontSize', ylabel_fontsize);
        xlabel('Profile number', 'FontSize', xlabel_fontsize);
        box on
        grid on
        hold off
        
        % Zoom on upper 50m water column
        sub(3) = subplot(1, 2, 2);
        hold on
        set(gca, 'YLimMode', 'manual', 'YDir', 'reverse', 'YLim', [0 50], ...
            'XLimMode', 'auto');
        set(gca, 'fontsize', axis_fontsize)
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        xlabel('Profile number', 'FontSize', xlabel_fontsize);
        % get required information about x-axis limits and ticks
        for i = 1:length(profile_data)
            temperature = profile_data{i}(:, 5);
            pressure = profile_data{i}(:, 7);
            % erase all invalid values
            valid_points = (pressure >= 0) & (pressure <= 50);
            temperature = temperature(valid_points);
            pressure = pressure(valid_points);
            x_axis = ones(size(pressure, 1), 1) * i;
            scatter(x_axis, pressure, 45, temperature, 'filled');
        end
        c = colorbar;
        cmocean('thermal');
        %colormap(gca, 'jet');
        %caxis([-5, 15]);
        c.Label.String = 'Temperature [C]';
        clear c
        
        
        % create main title
        if (p_float{1}.SECOND_ORDER_INFORMATION.best_guess_position_lat > 0)
            deployment_string = ' (Arctic)';
        else
            deployment_string = ' (Antarctica)';
        end
        t = title([ 'Float ' ...
               num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
               ' @ ' ...
               datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
               deployment_string ...
        ]);
        set(t, 'FontSize', title_fontsize);
        clear t
        
        box on
        grid on
        hold off
         
        set(gcf, 'Renderer', 'painters');
        print(figure_id, ...
         [directory p_config.PLOT_ASCENT.image_filename ...
           num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)] ...
         , image_type);
     
        if (p_config.PLOT_ASCENT.save_figure)
            set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
            saveas(figure_id, ...
                [directory_fig ...
                 p_config.PLOT_ASCENT.figure_filename ...
                 num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)]);
        end
        
        close(figure_id);
        clear figure_id f
        boolean = true;
        
    end

end

