function [ boolean ] = save_data( p_nemo_data, p_config )
%SAVE_DATA Writes the input nemo data using the given config.
%   This function saves given nemo data using nemo.pofile.save(). All
%   required options have to be stored in p_config.OUTPUT_FILES.
%
%   Returns: Boolean variable, being false if something went wrong during
%            write.
%
%   Parameters:
%   p_nemo_data:  Contains structure with all float and profile information.
%   p_config:     Contains all config options that are required, such as
%                 output path as well as filename format (currently not
%                 implemented) of profile files.
%   

%% Initialize return variables
boolean = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_data) || ~isstruct(p_config))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Save all profiles

output_folder = p_config.OUTPUT_FILES.output_path;
header_filename = p_config.OUTPUT_FILES.profile_header_file;

if (~exist(header_filename, 'file'))
    warning('Specified header file does not exist. Please check your configuration!');
    return;
end

float_count = length(p_nemo_data);

if (~exist(output_folder, 'dir'))
    if (~mkdir(output_folder))
        warning('Could not create output folder for nemo profiles!');
        return;
    end
end

for o_float = 1:float_count % every float will be processed
    profile_count = length(p_nemo_data{o_float});
    profile_subfoldername = sprintf('%04d', p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number);
    profile_folder = [output_folder profile_subfoldername p_config.GENERAL.folder_separator];
    
    % check if folder exists
    if (~exist(profile_folder, 'dir'))
        if (~mkdir(profile_folder))
            warning(['Could not create output subfolder for nemo float number ' profile_subfoldername '!']);
            return;
        end
    end
    
    for o_profile = 1:profile_count
        % construct desired profile filename
        profile_number = sprintf('%04d', p_nemo_data{o_float}{o_profile}.PROFILE_TECHNICAL_DATA.xmit_profile_number);
        if isnan(p_nemo_data{o_float}{o_profile}.PROFILE_TECHNICAL_DATA.xmit_profile_number)
            disp([mfilename ': Float ' profile_subfoldername ...
                ' contains a profile without xmit_profile_number!']);
            continue
        end
        output_filename = [profile_folder profile_subfoldername '_' ...
            profile_number '.profile'];
        
        overwrite_file = 1; % if true, existing files will be overwritten
        if (p_config.OUTPUT_FILES.update_changed_files_only) ...
                && exist(output_filename, 'file')
            % load existing profile
            old_profile = nemo.profile.load(output_filename);
            if nemo.profile.compare(old_profile, ...
                    p_nemo_data{o_float}{o_profile})
                % profiles are the same, so do not overwrite current one
                overwrite_file = 0;
            end
            clear old_profile
        end
        
        if ~nemo.profile.save(header_filename, ...
                p_nemo_data{o_float}{o_profile}, output_filename, overwrite_file)
            warning(['Writing profile ' profile_number ' of float with serial ' profile_subfoldername ' failed!' ]);
        end
    end
end

boolean = true;



end

